// InformationRepository.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;
import swarm.collections.ArrayImpl;

/** 
 * A static class used to distribute information among models
 *
 * @author Pietro Terna
 */
public class InformationRepository
{
    /** the use of computational steps */
    public static boolean computationalStepsInUse=false;

    /** the array of the models */
    public static ArrayImpl eSFrameModelSwarmArray;
    /** the arrays of unitList, unitListIndex, unitActivitySpace */
    public static ArrayImpl unitListArray, unitListIndexArray, unitActivitySpaceArray;
    /** the vector of interVisibilityMinLevel */
    public static double interVisibilityMinLevelVector[];
    /** the vector of uniqueAssignmentInEachCycle values */
    public static boolean uniqueAssignmentInEachCycleVector[];
    /** the vector of shuffleListsAtEachAssignment values */
    public static boolean shuffleListsAtEachAssignmentVector[];

    /** the vectors with model numbers */
    public static int modelNumbers[], modelNumbersRandomlyReordered[];

    /** the number of strata (models) in this run */
    public static int stratumNumber;

    /** the list of the hibernated units 
     *  this list is used to manage the virtually dropped units */
    public static ListImpl hibernatedUnitList;

    /** the list of the hibernated orders 
     *  this list is used to manage the virtually dropped orders */
    public static ListImpl hibernatedOrderList;

    /** the list of the hibernated memoryMatrixes 
     *  this list is used to manage the virtually dropped memoryMatrixes */
    public static ListImpl hibernatedMemoryMatrixList;

    /** the list of the hibernated  
     *  this list is used to manage the virtually dropped memoryMatrixes */
    public static ListImpl hibernatedComputationalSpecificationSetList;

    /** the list of the hibernated  
     *  this list is used to manage the virtually dropped memoryMatrixes */
    public static ListImpl hibernatedComputationalAssemblerList;

    /** the creation of the list memory spaces, once in a run, from the first model */
    public static void createArraysAndVectors(Zone aZone, int ln)
    {
	stratumNumber=ln;
	unitListArray = new ArrayImpl (aZone, stratumNumber);
	unitListIndexArray = new ArrayImpl (aZone, stratumNumber);
	unitActivitySpaceArray = new ArrayImpl (aZone, stratumNumber);
	interVisibilityMinLevelVector = new double[stratumNumber];
	uniqueAssignmentInEachCycleVector= new boolean[stratumNumber];
	shuffleListsAtEachAssignmentVector= new boolean[stratumNumber];
	hibernatedUnitList = new ListImpl (aZone);
	hibernatedOrderList = new ListImpl (aZone);
	hibernatedMemoryMatrixList = new ListImpl (aZone);
	hibernatedComputationalSpecificationSetList = new ListImpl (aZone);
	hibernatedComputationalAssemblerList = new ListImpl (aZone);

	modelNumbers = new int[stratumNumber];
	modelNumbersRandomlyReordered = new int[stratumNumber];
    }

    /** setting information, from the observer */
    public static void setInformationFromObserver(ArrayImpl modelA)
    {
	eSFrameModelSwarmArray = modelA;
    }

    /** setting information, from each model */
    public static void setInformationFromModel(int stratum, ListImpl unitList, 
					       ListIndex unitListIndex, 
					       UnitActivitySpace unitActivitySpace, 
					       double interVisibilityMinLevel,
					       boolean uniqueAssignmentInEachCycle,
					       boolean shuffleListsAtEachAssignment)
    {
	unitListArray.atOffset$put(stratum, unitList);
	unitListIndexArray.atOffset$put(stratum, unitListIndex);
	unitActivitySpaceArray.atOffset$put(stratum, unitActivitySpace);
	interVisibilityMinLevelVector[stratum]=interVisibilityMinLevel;
	uniqueAssignmentInEachCycleVector[stratum]=uniqueAssignmentInEachCycle;
	shuffleListsAtEachAssignmentVector[stratum]=shuffleListsAtEachAssignment;
    }

    /** shuffling in each cycle, from the first model schedule, the model order */
    public static void setModelSequence()
    {
	int i, ii, ir;

	for (i=0;i<stratumNumber;i++)modelNumbers[i]=i;

	for (i=stratumNumber-1;i>=0;i--)
	    {
		ir=Globals.env.uniformIntRand.getIntegerWithMin$withMax(0, i);
		modelNumbersRandomlyReordered[i]=modelNumbers[ir];
		for (ii=ir;ii<i;ii++)modelNumbers[ii]=modelNumbers[ii+1];
	    }

    }

    /** getting a specific unitList from random model sequence */
    public static Object getUnitListFromReorderedModelSequence(int stratum)
    {
	return unitListArray.atOffset(modelNumbersRandomlyReordered[stratum]);
    }

    /** getting a specific unitList from model sequence */
    public static Object getUnitListFromModelSequence(int stratum)
    {
	return unitListArray.atOffset(stratum);
    }

    /** getting a specific unitListIndex from random model sequence */
    public static Object getUnitListIndexFromReorderedModelSequence(int stratum)
    {
	return unitListIndexArray.atOffset(modelNumbersRandomlyReordered[stratum]);
    }

    /** getting a specific unitActivitySpace */
    public static Object getUnitActivitySpaceFromModel(int stratum)
    {
	return unitActivitySpaceArray.atOffset(stratum);
    }

    /** getting a specific interVisibilityMinLevel */
    public static double getInterVisibilityMinLevel(int stratum)
    {
	return interVisibilityMinLevelVector[stratum];
    }

    /** getting a specific uniqueAssignmentInEachCycle value */
    public static boolean getUniqueAssignmentInEachCycle(int stratum)
    {
	return uniqueAssignmentInEachCycleVector[stratum];
    }

    /** getting a specific shuffleListsAtEachAssignment value */
    public static boolean getShuffleListsAtEachAssignment(int stratum)
    {
	return shuffleListsAtEachAssignmentVector[stratum];
    }

    /** getting a specific model */
    public static Object getESFrameModelSwarm(int stratum)
    {
	return  eSFrameModelSwarmArray.atOffset(stratum);
    }

    /** getting the number of strata in this run */
    public static int getStratumNumber()
    {
	return stratumNumber;
    }

    /** getting the computationalStepsInUse status */
    public static boolean getComputationalStepsInUse()
    {
	return computationalStepsInUse;
    }

    /** setting the computationalStepsInUse status */
    public static void setComputationalStepsInUse(boolean s)
    {
	computationalStepsInUse=s;
    }

    /** getting randomly reordered numbers */
    public static int getModelNumbersRandomlyReordered(int stratum)
    {
	return modelNumbersRandomlyReordered[stratum];
    }

    /** adding a unit to the hibernatedUnitList */
    public static void addToHibernatedUnitList(Unit aUnit)
    {
	hibernatedUnitList.addLast(aUnit);
    }

    /** removing a unit from the hibernatedUnitList */
    public static Unit removeFromHibernatedUnitList()
    {
	return (Unit) hibernatedUnitList.removeFirst();
    }

    /** getting the count of the hibernatedUnitList */
    public static int getCountFromHibernatedUnitList()
    {
	return (int) hibernatedUnitList.getCount();
    }

    /** adding an order to the hibernatedOrderList */
    public static void addToHibernatedOrderList(Order anOrder)
    {
	hibernatedOrderList.addLast(anOrder);
    }

    /** removing an order from the hibernatedOrderList */
    public static Order removeFromHibernatedOrderList()
    {
	return (Order) hibernatedOrderList.removeFirst();
    }

    /** getting the count of the hibernatedOrderList */
    public static int getCountFromHibernatedOrderList()
    {
	return (int) hibernatedOrderList.getCount();
    }

    /** adding an order to the hibernatedMemoryMatrixList */
    public static void addToHibernatedMemoryMatrixList(MemoryMatrix aMemoryMatrix)
    {
	hibernatedMemoryMatrixList.addLast(aMemoryMatrix);
    }

    /** removing an order from the hibernatedMemoryMatrixList */
    public static MemoryMatrix removeFromHibernatedMemoryMatrixList()
    {
	return (MemoryMatrix) hibernatedMemoryMatrixList.removeFirst();
    }

    /** getting the count of the hibernatedMemoryMatrixList */
    public static int getCountFromHibernatedMemoryMatrixList()
    {
	return (int) hibernatedMemoryMatrixList.getCount();
    }

    /** adding an order to the hibernatedComputationalSpecificationSetList */
    public static void addToHibernatedComputationalSpecificationSetList(
				      ComputationalSpecificationSet aComputationalSpecificationSet)
    {
	hibernatedComputationalSpecificationSetList.addLast(aComputationalSpecificationSet);
    }

    /** removing an order from the hibernatedComputationalSpecificationSetList */
    public static ComputationalSpecificationSet 
	removeFromHibernatedComputationalSpecificationSetList()
    {
	return (ComputationalSpecificationSet) 
	    hibernatedComputationalSpecificationSetList.removeFirst();
    }

    /** getting the count of the hibernatedComputationalSpecificationSetList */
    public static int getCountFromHibernatedComputationalSpecificationSetList()
    {
	return (int) hibernatedComputationalSpecificationSetList.getCount();
    }

    /** adding an order to the hibernatedComputationalAssemblerList */
    public static void addToHibernatedComputationalAssemblerList(
				      ComputationalAssembler aComputationalAssembler)
    {
	hibernatedComputationalAssemblerList.addLast(aComputationalAssembler);
    }

    /** removing an order from the hibernatedComputationalAssemblerList */
    public static ComputationalAssembler 
	removeFromHibernatedComputationalAssemblerList()
    {
	return (ComputationalAssembler) 
	    hibernatedComputationalAssemblerList.removeFirst();
    }

    /** getting the count of the hibernatedComputationalAssemblerList */
    public static int getCountFromHibernatedComputationalAssemblerList()
    {
	return (int) hibernatedComputationalAssemblerList.getCount();
    }

}
